#
# Cookbook Name:: ec2dnsserver
# Recipe:: default
#
# Copyright 2013 EverTrue, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

fail 'This cookbook will only work on nodes in ec2' unless node['ec2']

package 'bind9'

include_recipe 'et_fog'
include_recipe "ec2dnsserver::#{node['ec2dnsserver']['log']['logger']}"

execute 'reload_zones' do
  command 'rndc reload'
  action :nothing
end

service node['ec2dnsserver']['service_name'] do
  supports status: true, restart: true, reload: true
  action :nothing
end

file '/etc/dhcp/dhclient-exit-hooks.d/set-bind-forwarders' do
  action :delete
end

log "ec2 hash: #{node['ec2'].inspect}" do
  level :info
end.run_action(:write)

forwarders = Ec2DnsServer.forwarders(node)

Chef::Log.info("Forwarders: #{forwarders}")

template "#{node['ec2dnsserver']['config_dir']}/named.conf.options" do
  source 'named.conf.options.erb'
  owner 'root'
  group 'root'
  mode 00644
  notifies :restart, "service[#{node['ec2dnsserver']['service_name']}]"
  variables(forwarders: forwarders)
end

template "#{node['ec2dnsserver']['config_dir']}/named.conf.local" do
  notifies :restart, "service[#{node['ec2dnsserver']['service_name']}]"
  variables(
    zones: node['ec2dnsserver']['zones'].reject do |_zone, zone_conf|
      zone_conf['type'] && zone_conf['type'] != 'master'
    end
  )
end

template "#{node['ec2dnsserver']['config_dir']}/named.conf.remote" do
  notifies :restart, "service[#{node['ec2dnsserver']['service_name']}]"
  variables(
    zones: node['ec2dnsserver']['zones'].select do |_zone, zone_conf|
      zone_conf['type'] && zone_conf['type'] != 'master'
    end
  )
end

cookbook_file "#{node['ec2dnsserver']['config_dir']}/named.conf" do
  notifies :restart, "service[#{node['ec2dnsserver']['service_name']}]"
end

node['ec2dnsserver']['zones'].each do |zone, zone_conf|
  Chef::Log.info("Zone #{zone} using suffix #{zone_conf['suffix']}") if zone_conf['suffix']

  ec2dnsserver_zone zone do
    vpcs zone_conf['vpcs'] if zone_conf['vpcs']
    stub zone_conf['stub'] if zone_conf['stub']
    ptr zone_conf['ptr_zone'] unless zone_conf['ptr_zone'].nil?
    suffix zone_conf['suffix'] if zone_conf['suffix']
    ns_zone zone_conf['ns_zone'] if zone_conf['ns_zone']
    static_records zone_conf['static_records'] if zone_conf['static_records']
    avoid_subnets node['ec2dnsserver']['avoid_subnets']
    contact_email node['ec2dnsserver']['contact_email']
    notifies :run, 'execute[reload_zones]'
  end
end

init_config_file = value_for_platform(
  %w(ubuntu debian) => { 'default' => '/etc/default/bind9' },
  %w(centos redhat suse fedora amazon amazonaws) => {
    'default' => '/etc/sysconfig/named'
  }
)

file init_config_file do
  if node['ec2dnsserver']['enable-ipv6']
    content "RESOLVCONF=no\nOPTIONS=-u bind\n"
  else
    content "RESOLVCONF=no\nOPTIONS=\"-u bind -4\n\""
  end
end

template '/etc/logrotate.d/named' do
  source 'logrotate.conf.erb'
  owner 'root'
  group 'root'
  mode 00644
end
